Screen {
    layout: vertical;
    background: #0d0b12;
    color: #f2f2f2;
}

#main {
    height: 1fr;
}

#left-panel {
    width: 1fr;
}

#right-panel {
    layout: vertical;
    width: 3fr;
    height: 1fr;
}

#detail-panel {
    height: 1fr;
}

#response-panel {
    height: 2fr;
}

RequestListWidget {
    border: solid #b5179e;
    background: #0d0b12;
    color: #f2f2f2;
}

RequestListWidget:focus {
    border: solid #f7b2ff;
}

DetailPanelWidget {
    border: solid #b5179e;
    padding: 0 1;
    background: #0d0b12;
    color: #f2f2f2;
}

DetailPanelWidget:focus {
    border: solid #f7b2ff;
}

ResponsePanelWidget {
    border: solid #b5179e;
    padding: 0 1;
    background: #0d0b12;
    color: #f2f2f2;
}

ResponsePanelWidget:focus {
    border: solid #f7b2ff;
}

ConfirmDeleteScreen {
    align: center middle;
    background: #0d0b12;
}

#confirm-dialog {
    layout: vertical;
    width: auto;
    min-width: 32;
    max-width: 60;
    height: auto;
    padding: 1 2;
    border: solid #b5179e;
    background: #1a0f1c;
    color: #f2f2f2;
    align: center middle;
}

#confirm-message {
    text-align: center;
    content-align: center middle;
}

#confirm-buttons {
    margin-top: 1;
    width: auto;
    height: auto;
    align: center middle;
}

#confirm-buttons Button {
    margin: 0 1;
}

Footer {
    background: #1a0f1c;
    color: #f2f2f2;
}

Footer .footer--key,
Footer .footer--highlight,
Footer .key {
    background: #b5179e;
    color: #0d0b12;
}
